/***
*
*	Copyright (c) 1996-2002, Valve LLC. All rights reserved.
*	
*	This product contains software technology licensed from Id 
*	Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc. 
*	All Rights Reserved.
*
*   Use, distribution, and modification of this source code and/or resulting
*   object code is restricted to non-commercial enhancements to products from
*   Valve LLC.  All other use, distribution, or modification is prohibited
*   without written permission from Valve LLC.
*
****/

/**
  * Codename HLDino
  * Version: Beta 1.0
  * Date:    Feb 2, 2005
  * Author:  fiNAL.Y
  * Description:
  * The source code for weapon_deserteagle
  * Copyright (C) 2000-2005 =M4CS= Development
  * Copyright (C) 2000-2005 SoftBoys Entertainment
  * Copyright (C) 2000-2005 MODChina.com
  */

#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "weapons.h"
#include "monsters.h"
#include "player.h"
#include "gamerules.h"


enum DESERTEAGLE_e {
	DESERTEAGLE_IDLE = 0,
	DESERTEAGLE_SHOOT1,
	DESERTEAGLE_SHOOT2,
	DESERTEAGLE_SHOOT_EMPTY,
	DESERTEAGLE_RELOAD,
	DESERTEAGLE_DRAW
};

const int DESERTEAGLE_DAMAGE = 30;

LINK_ENTITY_TO_CLASS( weapon_deserteagle, CDesertEagle );

int CDesertEagle::GetItemInfo(ItemInfo *p)
{
	p->pszName = STRING(pev->classname);
	p->pszAmmo1 = "deserteagle";
	p->iMaxAmmo1 = DESERTEAGLE_MAX_CARRY;
	p->pszAmmo2 = NULL;
	p->iMaxAmmo2 = -1;
	p->iMaxClip = DESERTEAGLE_MAX_CLIP;
	p->iFlags = 0;
	p->iSlot = 1;
	p->iPosition = 1;
	p->iId = m_iId = WEAPON_DESERTEAGLE;
	p->iWeight = DESERTEAGLE_WEIGHT;

	return 1;
}

int CDesertEagle::AddToPlayer( CBasePlayer *pPlayer )
{
	if ( CBasePlayerWeapon::AddToPlayer( pPlayer ) )
	{
		MESSAGE_BEGIN( MSG_ONE, gmsgWeapPickup, NULL, pPlayer->pev );
			WRITE_BYTE( m_iId );
		MESSAGE_END();
		return TRUE;
	}
	return FALSE;
}

void CDesertEagle::Spawn( )
{
	pev->classname = MAKE_STRING("weapon_deserteagle");
	Precache( );
	m_iId = WEAPON_DESERTEAGLE;
	SET_MODEL(ENT(pev), "models/w_deserteagle.mdl");

	m_iDefaultAmmo = DESERTEAGLE_DEFAULT_GIVE;

	FallInit();// get ready to fall down.
}


void CDesertEagle::Precache( void )
{
	PRECACHE_MODEL("models/v_deserteagle.mdl");
	PRECACHE_MODEL("models/w_deserteagle.mdl");
	PRECACHE_MODEL("models/p_deserteagle.mdl");

	PRECACHE_SOUND ("weapons/357_cock1.wav");
	PRECACHE_SOUND ("weapons/deserteagle_shot1.wav");
	PRECACHE_SOUND ("weapons/deserteagle_shot2.wav");

	m_usFireDesertEagle = PRECACHE_EVENT( 1, "events/DesertEagle.sc" );
}

BOOL CDesertEagle::Deploy( )
{
#ifdef CLIENT_DLL
	if ( bIsMultiplayer() )
#else
	if ( g_pGameRules->IsMultiplayer() )
#endif
	{
		// enable laser sight geometry.
		pev->body = 1;
	}
	else
	{
		pev->body = 0;
	}

	return DefaultDeploy( "models/v_deserteagle.mdl", "models/p_deserteagle.mdl", DESERTEAGLE_DRAW, "DesertEagle", UseDecrement(), pev->body );
}

void CDesertEagle::PrimaryAttack()
{
	if (m_iClip <= 0)
	{
		PlayEmptySound();
		SendWeaponAnim( DESERTEAGLE_SHOOT_EMPTY );
		m_flNextPrimaryAttack = 0.15;
		return;
	}

	m_pPlayer->m_iWeaponVolume = LOUD_GUN_VOLUME;
	m_pPlayer->m_iWeaponFlash = BRIGHT_GUN_FLASH;

	m_iClip--;

	m_pPlayer->pev->effects = (int)(m_pPlayer->pev->effects) | EF_MUZZLEFLASH;

	// player "shoot" animation
	m_pPlayer->SetAnimation( PLAYER_ATTACK1 );

	Vector vecSrc	 = m_pPlayer->GetGunPosition( );
	Vector vecAiming = m_pPlayer->GetAutoaimVector( AUTOAIM_10DEGREES );

	Vector vecDir;
	vecDir = m_pPlayer->FireBulletsPlayer( 1, vecSrc, vecAiming, VECTOR_CONE_1DEGREES, 8192, BULLET_PLAYER_357, 0, DESERTEAGLE_DAMAGE, m_pPlayer->pev, m_pPlayer->random_seed );

    int flags;
#if defined( CLIENT_WEAPONS )
	flags = FEV_NOTHOST;
#else
	flags = 0;
#endif

	PLAYBACK_EVENT_FULL( flags, m_pPlayer->edict(), m_usFireDesertEagle, 0.0, (float *)&g_vecZero, (float *)&g_vecZero, vecDir.x, vecDir.y, 0, 0, 0, 0 );

	if (!m_iClip && m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] <= 0)
		// HEV suit - indicate out of ammo condition
		m_pPlayer->SetSuitUpdate("!HEV_AMO0", FALSE, 0);

	m_flNextPrimaryAttack = 0.75;
	m_flTimeWeaponIdle = UTIL_SharedRandomFloat( m_pPlayer->random_seed, 10, 15 );
}


void CDesertEagle::Reload( void )
{
	if ( m_pPlayer->ammo_deserteagle <= 0 )
		return;

	DefaultReload( DESERTEAGLE_MAX_CLIP, DESERTEAGLE_RELOAD, 1.5);
}


void CDesertEagle::WeaponIdle( void )
{
	ResetEmptySound( );

	if (m_flTimeWeaponIdle > UTIL_WeaponTimeBase() )
		return;
	
	SendWeaponAnim( DESERTEAGLE_IDLE );

	m_flTimeWeaponIdle = UTIL_SharedRandomFloat( m_pPlayer->random_seed, 10, 15 );
}



class CDesertEagleAmmo : public CBasePlayerAmmo
{
	void Spawn( void )
	{ 
		Precache( );
		SET_MODEL(ENT(pev), "models/w_357ammobox.mdl");
		CBasePlayerAmmo::Spawn( );
	}
	void Precache( void )
	{
		PRECACHE_MODEL ("models/w_357ammobox.mdl");
		PRECACHE_SOUND("items/9mmclip1.wav");
	}
	BOOL AddAmmo( CBaseEntity *pOther ) 
	{ 
		if (pOther->GiveAmmo( AMMO_DESERTEAGLE_GIVE, "deserteagle", DESERTEAGLE_MAX_CARRY ) != -1)
		{
			EMIT_SOUND(ENT(pev), CHAN_ITEM, "items/9mmclip1.wav", 1, ATTN_NORM);
			return TRUE;
		}
		return FALSE;
	}
};
LINK_ENTITY_TO_CLASS( ammo_deserteagle, CDesertEagleAmmo );
